import 'dart:async';

typedef T OnData<T>(T data);

class StateStream<T> implements Stream<T> {
  final Stream<T> stream;
  final T initialData;

  StateStream(this.stream, this.initialData);

  @override
  Future<bool> any(bool Function(T element) test) => stream.any(test);

  @override
  StateStream<T> asBroadcastStream(
          {void Function(StreamSubscription<T> subscription) onListen,
          void Function(StreamSubscription<T> subscription) onCancel}) =>
      StateStream(stream.asBroadcastStream(onListen: onListen, onCancel: onCancel), initialData);

  @override
  StateStream<E> asyncExpand<E>(Stream<E> Function(T event) convert) =>
      StateStream<E>(stream.asyncExpand(convert), initialData as E);

  @override
  StateStream<E> asyncMap<E>(FutureOr<E> Function(T event) convert) =>
      StateStream(stream.asyncMap(convert), convert(initialData));

  @override
  StateStream<R> cast<R>() => StateStream(stream.cast<R>(), initialData as R);

  @override
  Future<bool> contains(Object needle) => stream.contains(needle);

  @override
  Future<E> drain<E>([E futureValue]) => stream.drain(futureValue);

  @override
  StateStream<T> distinct([bool Function(T previous, T next) equals]) =>
      StateStream(stream.distinct(equals), initialData);

  @override
  StateStream<T> handleError(Function onError, {bool Function(Error error) test}) =>
      StateStream(stream.handleError(onError, test: test), initialData);

  @override
  StateStream<S> map<S>(S Function(T event) convert) =>
      StateStream(stream.map(convert), convert(initialData));

  @override
  StateStream<T> skip(int count) => StateStream(stream.skip(count), initialData);

  @override
  StateStream<T> skipWhile(bool Function(T element) test) =>
      StateStream(stream.skipWhile(test), initialData);

  @override
  StateStream<T> take(int count) => StateStream(stream.take(count), initialData);

  @override
  StateStream<T> takeWhile(bool Function(T element) test) =>
      StateStream(stream.takeWhile(test), initialData);

  @override
  StateStream<T> timeout(Duration timeLimit, {void Function(EventSink<T> sink) onTimeout}) =>
      StateStream(stream.timeout(timeLimit, onTimeout: onTimeout), initialData);

  @override
  StateStream<S> transform<S>(StreamTransformer<T, S> streamTransformer) =>
      StateStream(stream.transform(streamTransformer), initialData as S);

  @override
  StateStream<T> where(bool Function(T event) test) => StateStream(stream.where(test), initialData);

  @override
  StateStream<S> expand<S>(Iterable<S> Function(T element) convert) =>
      StateStream(stream.expand(convert), initialData as S);

  @override
  Future<T> elementAt(int index) => stream.elementAt(index);

  @override
  Future<bool> every(bool Function(T element) test) => stream.every(test);

  @override
  Future<T> get first => stream.first;

  @override
  Future<T> firstWhere(bool Function(T element) test, {T Function() orElse}) =>
      stream.firstWhere(test, orElse: orElse);

  @override
  Future<S> fold<S>(S initialValue, S Function(S previous, T element) combine) =>
      stream.fold(initialValue, combine);

  @override
  Future forEach(void Function(T element) action) => stream.forEach(action);

  @override
  bool get isBroadcast => stream.isBroadcast;

  @override
  Future<bool> get isEmpty => stream.isEmpty;

  @override
  Future<String> join([String separator = ""]) => stream.join(separator);

  @override
  Future<T> get last => stream.last;

  @override
  Future<T> lastWhere(bool Function(T element) test, {T Function() orElse}) =>
      stream.lastWhere(test, orElse: orElse);

  @override
  Future<int> get length => stream.length;

  @override
  Future pipe(StreamConsumer<T> streamConsumer) => stream.pipe(streamConsumer);

  @override
  Future<T> reduce(T Function(T previous, T element) combine) => stream.reduce(combine);

  @override
  Future<T> get single => stream.single;

  @override
  Future<T> singleWhere(bool Function(T element) test, {T Function() orElse}) =>
      stream.singleWhere(test, orElse: orElse);

  @override
  Future<List<T>> toList() => stream.toList();

  @override
  Future<Set<T>> toSet() => stream.toSet();

  @override
  StreamSubscription<T> listen(void Function(T event) onData,
          {Function onError, void Function() onDone, bool cancelOnError}) =>
      stream.listen(onData, onError: onError, onDone: onDone, cancelOnError: cancelOnError);
}
